// Author: Heinrich Moser, 9925866, http://www.heinzi.at
//
// Uses SOAP to access the google web service
//
// See googleclient.html for a more detailed description

package at.heinzi.googleclient;

import java.io.*;
import javax.xml.soap.*;
import javax.xml.parsers.*;
import javax.xml.messaging.*;
import javax.xml.transform.*;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import org.w3c.dom.*;

public class XMLClient {

    public static void main(String[] args) {
        try {
            // Create new search request
            SearchRequest sr = new SearchRequest();
            
            // Get search terms
            System.out.print("Search terms: ");
            try {
                sr.queryTerms = (new BufferedReader(new InputStreamReader(System.in))).readLine();
            } catch (IOException ex) {
                ex.printStackTrace();
                System.exit(1);
            }
            
            // Send request and get response
            SOAPMessage response = sr.send();
            
            // Print XML
            (new XMLPrettyWriter(response)).print("SOAP Response");
            
            // Print response
            (new SearchResponse(response)).printResults();
            
        } catch (SOAPException ex) {
            System.err.println("SOAPException: " + ex.getMessage());
        }
    }
}

// Represents a search response
class SearchResponse {

    private Document p_xml;

    // Constructor
    public SearchResponse(SOAPMessage msg) {
        this.p_xml = XMLPrettyWriter.soap2xml(msg);
    }

    // Print search results
    public void printResults() {
        Element soapenv = this.p_xml.getDocumentElement();
        Element soapbody = (Element)soapenv.getElementsByTagName("SOAP-ENV:Body").item(0);
        Element resp = (Element)soapbody.getElementsByTagName("ns1:doGoogleSearchResponse").item(0);
        Element ret = (Element)resp.getElementsByTagName("return").item(0);
        Element resultElements = (Element)ret.getElementsByTagName("resultElements").item(0);
        NodeList lst = resultElements.getElementsByTagName("item");

        // Loop through all results
        for (int i = 0; i < lst.getLength(); i++) {
            Element re = (Element)lst.item(i);
            String title = re.getElementsByTagName("title").item(0).getFirstChild().getNodeValue();
            String url = re.getElementsByTagName("URL").item(0).getFirstChild().getNodeValue();

            System.out.println(title);
            System.out.println(url);
            System.out.println("");
        }
    }
}

// Represents a generic request
abstract class Request {

    static final String SOAP_URL = "http://api.google.com/search/beta2";
    static final String KEY = "Jy5IbvlQFHJsV+Nz/azR4K4Le7O+s9hq";
    protected SOAPMessage p_msg;

    // Construct basic request
    protected SOAPBodyElement createRequestMessage(String bodyElement) throws SOAPException {
        System.out.println("Create empty SOAP-Message...");

        // Create SOAP message
        this.p_msg = MessageFactory.newInstance().createMessage();

        // Show empty message
        (new XMLPrettyWriter(this.p_msg)).print("Empty Message");

        // Remove header (not necessary)
        SOAPEnvelope env = this.p_msg.getSOAPPart().getEnvelope();
        env.getHeader().detachNode();

        // Add bodyElement (doGoogleSearch, ...) (gns = GoogleNameSpace)
        SOAPBodyElement be = env.getBody().addBodyElement(env.createName(bodyElement, "gns", "urn:GoogleSearch"));
        be.setEncodingStyle("http://schemas.xmlsoap.org/soap/encoding/");

        // Set key
        be.addChildElement("key").addTextNode(KEY);
        
        // Save changes
        this.p_msg.saveChanges();

        // Return body element
        return be;
    }

    // Send the complete request
    public SOAPMessage send() throws SOAPException {
        SOAPConnection scn;

        System.out.println("Open SOAP connection...");

        // Create point-to-point connection to Google
        scn = SOAPConnectionFactory.newInstance().createConnection();

        // Print request
        (new XMLPrettyWriter(this.p_msg)).print("Request Message");

        // Send request
        System.out.println("Send message and retrieve result...");
        SOAPMessage response = scn.call(this.p_msg, new URLEndpoint(this.SOAP_URL));

        // Close the connection
        System.out.println("Close connection...");
        scn.close();

        return response;
    }
}

// Represents a Google search request
class SearchRequest extends Request {

    public String queryTerms = "";
    public int start = 1;
    public int maxResults = 10;
    public boolean filter = false;
    public String restrict = "";
    public boolean safeSearch = false;
    public String lr = ""; // language restrictions
    
    // Send the request and retrieve the result
    public SOAPMessage send() throws SOAPException {

        // Create doGoogleSearch message
        SOAPBodyElement be = createRequestMessage("doGoogleSearch");
        
        // Set parameters
        be.addChildElement("q").addTextNode(this.queryTerms);
        be.addChildElement("start").addTextNode(Integer.toString(start));
        be.addChildElement("maxResults").addTextNode(Integer.toString(maxResults));
        be.addChildElement("filter").addTextNode(Boolean.toString(filter));
        be.addChildElement("restrict").addTextNode(restrict);
        be.addChildElement("safeSearch").addTextNode(Boolean.toString(safeSearch));
        be.addChildElement("lr").addTextNode(lr);
        be.addChildElement("ie").addTextNode(""); // deprecated
        be.addChildElement("oe").addTextNode(""); // deprecated
        this.p_msg.saveChanges();

        // send request
        SOAPMessage response = super.send();
        return response;
    }
}
        
// Do SOAP2XML and XML2PrettyText conversions
class XMLPrettyWriter {
    Document p_xml;

    // Convert a SOAPMessage into an XML document
    public static Document soap2xml(SOAPMessage msg) {
        try {
            ByteArrayOutputStream s = new ByteArrayOutputStream();
            msg.writeTo(s);
            return DocumentBuilderFactory.newInstance().newDocumentBuilder().parse(new ByteArrayInputStream(s.toByteArray()));
        } catch (IOException ex) {
            System.err.println("IOException: " + ex.getMessage());
            System.exit(1);
            return null;
        } catch (ParserConfigurationException ex) {
            System.err.println("ParserConfigurationException: " + ex.getMessage());
            System.exit(1);
            return null;
        } catch (org.xml.sax.SAXException ex) {
            System.err.println("org.xml.sax.SAXException: " + ex.getMessage());
            System.exit(1);
            return null;
        } catch (SOAPException ex) {
            System.err.println("SOAPException: " + ex.getMessage());
            System.exit(1);
            return null;
        }
    }

    // Constructor
    public XMLPrettyWriter(SOAPMessage msg) {
        this.p_xml = this.soap2xml(msg);
    }

    // Output the XML document as
    // * raw XML
    // * hierarchical output
    public void print(String msg) {
        System.out.println("------\n" + msg + "\n");
        
        // Write raw XML
        try {
            TransformerFactory.newInstance().newTransformer().transform(new DOMSource(this.p_xml),
                                                                        new StreamResult(System.out));
        } catch (TransformerException ex) {
            System.err.println("TransformerException: " + ex.getMessage());
            System.exit(1);
        }
        System.out.println("\n");
        
        // Write hierarchical
        this.p_printNode(0, this.p_xml.getDocumentElement());
        
        System.out.println("------");
    }

    // Write the node and its child nodes (recursive)
    // Indent depending on the level
    private void p_printNode(int level, org.w3c.dom.Node n) {
        
        // Indentation
        StringBuffer indent = new StringBuffer();
        for (int i = 0; i < level; i++) {
            indent.append("  ");
        }

        // Beginning of node
        if (n instanceof Element) {
            System.out.print(indent.toString() + "- " + ((Element)n).getTagName()); // Tag name

            // Data type
            NamedNodeMap atts = n.getAttributes();
            org.w3c.dom.Node type = atts.getNamedItem("xsi:type");
            System.out.println((type == null) ? "" : (" (" + type.getNodeValue() + ")"));
            
            // Attributes
            for (int i = 0; i < atts.getLength(); i++) {
                Attr a = (Attr)atts.item(i);
                if (!a.getName().equals("xsi:type")) {
                    System.out.println(indent.toString() + "  A " + a.getName() + " = " + a.getValue());
                }
            }
        } else {
            if (!(n.getNodeName().equals("#text") && n.getNodeValue().equals("\n"))) {
                System.out.println(indent.toString() + n.getNodeName() + ": " + n.getNodeValue()); // Generic node
            }
        }

        // Write child nodes
        NodeList nlst = n.getChildNodes();
        for (int i = 0; i < nlst.getLength(); i++) {
            this.p_printNode(level + 1, nlst.item(i));
        }
    }
}
